#!/usr/bin/env python

##
#   Author: IL'dar AKHmetgaleev aka AkhIL
#   e-mail: akhilman at gmail dot com
#   web:    http://akhil.nm.ru
#
#   This code is licensed under 
#   Creative Commons Attribution 3.0 Unported License
#   http://creativecommons.org/licenses/by/3.0/
##

version = "20080209"

import xml.dom.minidom as minidom
import sys,os
import gzip
import re
import string

def printhelp ():
    sys.stdout.write("""\
%(name)s v%(version)s renders sif to multiple files for each layer.
To define layer you should put <layer name> to layer node description.
    eg. "Fat Man <layer_one>"
%(name)s will create separate directories for each rendering layer.

Usage:
    %(name)s mymega.sifz [more options]

Options:
    -l --list           print exitst rendering layers
    --ignore <list>     coma separated list of layer which shouldn't be rendered    
    --split             only split sif to layer sifs and not render it

    for more options run synfig --help

"""%{
    "name":os.path.basename(sys.argv[0]),
    "version":version
    })


def get_sifdom (siffile):
    if not siffile:
        printhelp()
        sys.exit()

    if os.path.splitext(siffile)[1] == ".sifz":
        sif = gzip.open(siffile).read()
    else:
        sif = file(siffile).read()

    sifdom = minidom.parseString(sif)
    return sifdom

def get_renderlayers (sifdom):
    renderlayers = []
    pattern = re.compile(".*<(.*)>.*")
    for layer in sifdom.getElementsByTagName("layer"):
        if layer.hasAttribute("desc"):
            if layer.getAttribute("active") == "true": # if layer visible in source
                match = pattern.search(layer.getAttribute("desc"))
                if match:
                    if match.groups()[0] not in renderlayers:
                        renderlayers.append(match.groups()[0])
    return renderlayers

def make_renderlayer_solo (sifdom, renderlayer):
    pattern = re.compile(".*<(.*)>.*")
    for layer in sifdom.getElementsByTagName("layer"):
        if layer.hasAttribute("desc"):
            if layer.getAttribute("active") == "true": # if layer visible in source
                match = pattern.search(layer.getAttribute("desc"))
                if match:
                    if match.groups()[0] == renderlayer:
                        layer.setAttribute("active","true")
                    else:
                        layer.setAttribute("active","false")

def main():
    siffile = ""
    output = ""
    ignorelayers = []
    listonly = False
    splitonly = False

    for arg in sys.argv[1:]:
        if arg not in sys.argv[1:]:
            continue
        n = sys.argv.index(arg)
        if arg in ("-h","--help"):
            printhelp()
            sys.exit()
        elif arg in ("-l","--list"):
            listonly = True
        elif arg in ("--split"):
            splitonly = True
        elif ".sif" in arg:
            siffile = sys.argv.pop(n)
        elif arg == "-o":
            sys.argv.pop(n)
            output = sys.argv.pop(n)
        elif arg == "--ignore":
            sys.argv.pop(n)
            ignorelayers = sys.argv.pop(n).split(",")

    sifdom = get_sifdom(siffile)
    renderlayers = get_renderlayers(sifdom)
    del sifdom

    if not renderlayers:
        sys.stderr.write("Can't find rendering layers in this sif!\n")
        sys.exit(1)

    if listonly:
        sys.stdout.write("rendering layers:\n")
        for rl in renderlayers:
            sys.stdout.write("%s\n"%rl)
        sys.exit()

    outsifs = []
    for rl in renderlayers:
        if rl in ignorelayers:
            continue
        sifdom = get_sifdom(siffile)
        make_renderlayer_solo(sifdom,rl)
        outsif = "%s.%s.sifz" % ( \
                                 os.path.splitext(siffile)[0], \
                                 rl.replace(" ","_") \
                                )
        outsifs.append(outsif)
        outsif = gzip.open(outsif, "w")
        outsif.write(sifdom.toxml())
        outsif.close()
        del sifdom

    if splitonly:
        sys.exit()

    for rl in renderlayers:
        layersif = "%s.%s.sifz" % ( \
                               os.path.splitext(siffile)[0], \
                               rl.replace(" ","_") \
                              )
        args = ""
        args += layersif
        args += " "
        args += string.join(sys.argv[1:]," ")
        args += " "
        if output:
            outimg = "%(prefix)s%(sep)s%(layer)s%(sep)s%(layer)s%(ext)s" % { \
                                    "prefix":os.path.splitext(os.path.basename(output))[0], \
                                    "layer":rl.replace(" ","_"), \
                                    "ext":os.path.splitext(output)[1], \
                                    "sep":os.path.sep
                                 }
            args += "-o %s" % outimg
            if not os.path.isdir(os.path.dirname(outimg)): 
                os.makedirs(os.path.dirname(outimg))
        sys.stderr.write('Rendering layer "%s" (%i/%i)\n' % (rl,renderlayers.index(rl)+1,len(renderlayers)))
        sys.stderr.write("$ synfig %s\n" % args )
        status = os.system("synfig %s" % args)
        if status:
            map(lambda x: os.unlink(x), outsifs)
            sys.exit(status)
    map(lambda x: os.unlink(x), outsifs)

main()
